# -*- coding: utf-8 -*-
# Copyright (c) 2010-2013 Infrae. All rights reserved.
# See also LICENSE.txt

from infrae.testing import suite_from_package, TestCase
from infrae.wsgi.testing.layer import BrowserLayer
from infrae.wsgi.testing.request import TestRequest
try:
    from infrae.wsgi.testing.intercept import (
        ZopeBrowserLayer, ZopeBrowser, http)
except ImportError:
    pass



# -*- coding: utf-8 -*-
# Copyright (c) 2010-2013 Infrae. All rights reserved.
# See also LICENSE.txt

from cStringIO import StringIO

from Acquisition import aq_base
from ZPublisher.BaseRequest import RequestContainer
from zope.interface import implements, alsoProvides

from infrae.wsgi.interfaces import ITestRequest
from infrae.wsgi.publisher import WSGIRequest, WSGIResponse
from infrae.wsgi.tests.mockers import MockRoot
from infrae.wsgi.utils import split_path_info

import urlparse
import urllib


class TestRequest(WSGIRequest):
    """Test request that inherit from the real request.
    """
    implements(ITestRequest)

    def __init__(self, application=None, layers=[], headers={},
                 url='http://localhost', method='GET', debug=True, form={}):
        url_parts = urlparse.urlparse(url)
        netloc_parts = url_parts.netloc.split(':')
        if len(netloc_parts) > 1:
            hostname = netloc_parts[0]
            port = netloc_parts[1]
        else:
            hostname = netloc_parts[0]
            if url_parts.scheme == 'https':
                port = '443'
            else:
                port = '80'
        query = url_parts.query
        if form:
            if isinstance(form, dict):
                data = []
                for key, value in form.iteritems():
                    if isinstance(value, (list, tuple)):
                        for item in value:
                            data.append((key, item))
                    else:
                        data.append((key, value))
            elif isinstance(form, (list, tuple)):
                data = form
            else:
                raise ValueError("Cannot encode form", form)
            query = urllib.urlencode(data)
        environ = {'SERVER_PROTOCOL': 'HTTP/1.0',
                   'SERVER_NAME': hostname,
                   'SERVER_PORT': port,
                   'wsgi.version': (1,0),
                   'wsgi.url_scheme': url_parts.scheme,
                   'wsgi.input': StringIO(),
                   'wsgi.errors': StringIO(),
                   'wsgi.multithread': False,
                   'wsgi.multiprocess': False,
                   'wsgi.run_once': False,
                   'wsgi.handleErrors': not debug,
                   'REQUEST_METHOD': method,
                   'SCRIPT_NAME': '',
                   'PATH_INFO': url_parts.path,
                   'QUERY_STRING': query}
        for name, value in headers:
            http_name = ('HTTP_' + name.upper()).replace('-', '_')
            environ[http_name] = value
        WSGIRequest.__init__(
            self,
            environ['wsgi.input'],
            environ,
            WSGIResponse(environ, lambda status, headers: None, debug))
        if application is None:
            application = MockRoot()
        else:
            application = aq_base(application.getPhysicalRoot())
        self.application = application.__of__(RequestContainer(REQUEST=self))
        self['PARENTS'] = [self.application,]
        for layer in layers:
            alsoProvides(self, layer)
        self.processInputs()
        path_info = self['PATH_INFO']

        path = list(reversed(split_path_info(path_info)))
        self['ACTUAL_URL'] = self['URL'] + urllib.quote(path_info)
        self['TraversalRequestNameStack'] = self.path = path
        method = self.get('REQUEST_METHOD', 'GET').upper()
        if method in ['GET', 'POST']:
            method = 'index_html'
        self.method = method

    def _hold(self, callback):
        # Discard cleanup callback
        pass



# -*- coding: utf-8 -*-
# Copyright (c) 2013  Infrae. All rights reserved.
# See also LICENSE.txt

import raven
import raven.utils.wsgi as wsgi_utils
from .log import object_name, object_path


class RavenLoggingPlugin(object):

    def __init__(self, raven_config):
        self.client = raven.Client(**raven_config)

    def __call__(self, request, response, obj, error_info,
                 short_message, full_traceback, extra):
        raven_data = {
            'message': "".join(short_message),
            'sentry.interfaces.Http': {
                'url': request.get('URL', 'n/a'),
                'method': request.environ.get('REQUEST_METHOD', 'n/a'),
                'query_string': request.environ.get('QUERY_STRING', 'n/a'),
                'headers': dict(wsgi_utils.get_headers(request.environ)),
                'env': dict(wsgi_utils.get_environ(request.environ))
                }}
        raven_extra = {
            'User':  request.get('AUTHENTICATED_USER', 'n/a') or 'n/a'}
        if obj is not None:
            raven_extra.update({
                    'Object Class': object_name(obj),
                    'Object Name': object_path(obj),
                    })
        if extra is not None:
            raven_extra.update({
                    'Extra Information': extra
                    })

        self.client.captureException(
            exc_info=error_info, data=raven_data, extra=raven_extra)



# -*- coding: utf-8 -*-
# Copyright (c) 2010-2013 Infrae. All rights reserved.
# See also LICENSE.txt

# This package define a simple infrae.testing layer with a wsgi
# application.

import re
import base64

from AccessControl.SecurityManagement import (
    getSecurityManager, setSecurityManager, noSecurityManager)
from zope.site.hooks import getSite, setSite, setHooks
from transaction import commit

from infrae.testing import Zope2Layer
from infrae.wsgi.publisher import WSGIApplication


basicre = re.compile('Basic (.+)?:(.+)?$')
def auth_header(header):
    """This function takes an authorization HTTP header and encode the
    couple user, password into base 64 like the HTTP protocol wants
    it.
    """
    match = basicre.match(header)
    if match:
        u, p = match.group(1, 2)
        if u is None:
            u = ''
        if p is None:
            p = ''
        auth = base64.encodestring('%s:%s' % (u, p))
        return 'Basic %s' % auth[:-1]
    return header


class TestBrowserWSGIResult(object):
    """Call a WSGI Application and return its result.

    Backup the ZCA local site before calling the wrapped application and
    restore it when done.
    """

    def __init__(self, app, connection, environ, start_response):
        self.app = app
        self.connection = connection
        self.environ = environ
        self.start_response = start_response
        self.__result = None
        self.__next = None
        self.__site = None
        self.__security = None

    def __iter__(self):
        return self

    def next(self):
        if self.__next is None:
            # Backup ZCA site and security manager
            self.__site = getSite()
            self.__security = getSecurityManager()
            noSecurityManager()
            self.__result = self.app(self.environ, self.start_response)
            self.__next = iter(self.__result).next
        return self.__next()

    def close(self):
        if self.__result is not None:
            if hasattr(self.__result, 'close'):
                self.__result.close()
        if self.__site is not None:
            setSite(self.__site)
            setHooks()
        if self.__security is not None:
            setSecurityManager(self.__security)
        self.connection.sync()


class TestWSGIMiddleware(object):
    """Simple middleware that commit changes before calling the
    application and restore the Zope status after. It is not
    compatible with zope.app.testing.
    """

    def __init__(self, app, connection):
        self.app = app
        self.connection = connection

    def __call__(self, environ, start_response):
        # Handle authorization
        auth_key = 'HTTP_AUTHORIZATION'
        if environ.has_key(auth_key):
            environ[auth_key] = auth_header(environ[auth_key])

        commit()
        return TestBrowserWSGIResult(
            self.app, self.connection, environ, start_response)


class BrowserLayer(Zope2Layer):
    """Functional test layer.
    """

    def _create_wsgi_application(self):
        # Create Zope WSGI Application. You can change this method if
        # you whish to change the tested WSGI application.
        return WSGIApplication(
            self._application, self._transaction_manager)

    def testSetUp(self):
        super(BrowserLayer, self).testSetUp()
        wsgi_app = self._create_wsgi_application()

        self._test_wsgi_application = TestWSGIMiddleware(
            wsgi_app, self._test_connection)

    def testTearDown(self):
        self._test_wsgi_application = None
        super(BrowserLayer, self).testTearDown()
